# Superiority and inferiority ranking (SIR) method
#
# parameters
#   PM - performance matrix (alternatives in rows, criteria in columns)
#   w - weights
#   d - preference function array
#   minmax - min/max value or vector of mixed values for criteria orientation
#   indifferenceTreshold list of indifference thresholds
#   prefferenceThreshold list of prefference thresholds
#   intermediateThreshold lish of intermediate thresholds for Gaussian function type
#   SAW  - if set to T, SIR-SAW is used as aggregation procedure, otherwise SIR-TOPSIS is used
#   VERBOSE - if set to T dump output to console
SIR <- function(PM, w, d, minmax = 'max', indifferenceTreshold = NULL, prefferenceThreshold = NULL,
                intermediateThreshold = NULL, SAW = TRUE, VERBOSE = FALSE){
  ## check validity of the objects manipulated by the current function
  # if there are less than 2 criteria or 2 alternatives, there is no MCDA problem
  if (is.null(dim(PM))) stop('less than 2 criteria or 2 alternatives')
  if (!(is.matrix(PM) || (is.data.frame(PM)))) stop('wrong performance matrix, should be a matrix or a data frame')
  if (!is.numeric(unlist(PM))) stop('Only numeric values in performance matrix expected')
  PM <- util_pm_minmax(PM, minmax) #validate minmax and invert scales if neccessary
  if (!is.data.frame(PM)) PM <- as.data.frame(PM)
  ncri <- ncol(PM)  #no. of criteria
  qj   <- indifferenceTreshold
  pj   <- prefferenceThreshold
  sj   <- intermediateThreshold
  if (!(is.vector(w))) stop('criteria weights should be a vector')
  if (!is.numeric(w)) stop('criteriaWeights should be a numeric vector')
  if (sum(w) != 1) stop('Sum of weights is expected to be 1')
  for(w1 in w){
    if(w1 < 0) stop('all weights must be greater then or equal to 0')
  }
  if (ncol(PM)!=length(w)) stop("length of criteriaWeights should be checked")
  fTypes <- c('default', 'U-shape', 'V-shape', 'level', 'linear', 'Gaussian')
  indifTypes <- c('U-shape', 'level', 'linear')
  indifTypesVector <- rep(F, times=ncri)
  prefTypes <- c('V-shape', 'level', 'linear')
  prefTypesVector <- rep(F, times=ncri)
  if (!(is.vector(d))) stop('preferenceFunction must be a vector')
  for(i in 1:ncri){ #check for consistency of preference function
    if(!(d[i] %in% fTypes)) stop('unsupported type of preference function used only: default, U-shape, V-shape, level, linear, Gaussian supported.')
    if(d[i] %in% indifTypes) indifTypesVector[i] <- T
    if(d[i] %in% prefTypes) prefTypesVector[i] <- T
  }
  if (T %in% prefTypesVector){ # prefference threshold required, check its consistency
    if(is.null(pj)) stop('some choosen preference functions need to have set prefference threshold.')
    if (!is.numeric(pj)) stop('prefferenceThreshold must be a numeric vector')
    if (length(pj) != ncri) stop('number of elements in prefferenceThreshold must be equal to number of criteria (set value to 0 for criteria with preference function that does not require this parameter).')
    for(i in 1:ncri){
      if (prefTypesVector[i] && pj[i] < 0){
        msg <- cat('prefference threshold for criterion ', i, ' set < 0, only non negative values accepted.')
        stop(msg)
      }
    }
  }
  if('Gaussian' %in% d){ # Gaussian type of prefference function check intermediate threshold for consistency
    if (is.null(sj) && is.null(pj) && is.null(qj)) stop('intermediate threshold required (and not possible to derive from indiference and prefference thresholds).')
    if (is.null(sj)) { # try to derive sj from other thresholds
      if (!is.numeric(pj) || !is.numeric(qj) || length(pj) != ncri || length(qj) != ncri) stop('intermediate threshold required (failed to derive it form other thresholds due to its inconsistencies)')
      sj <- (pj + qj)/2
      if (length(sj[sj < 0]) > 0) stop('intermediate threshold required (failed to derive it form other thresholds due to its inconsistencies)')
    }
  }
  #clean-up (remove variables used only for consistency check)
  fTypes     <- NULL
  indifTypes <- NULL
  prefTypes  <- NULL
  indifTypesVector <- NULL
  prefTypesVector  <- NULL
  w1 <- NULL
  ## End of checking the validity of the "inputs"

  nalt <- nrow(PM)  #no. of alternatives
  alt  <- rownames(PM)
  cri  <- colnames(PM)

  #pairwaise comparison (validated in PROMETHEE)
  DK <- list()
  for(k in 1:ncri) {
    DKf <- matrix(data = 0, nrow = nalt, ncol = nalt)
    for(i in 1:nalt) {
      for(j in 1:nalt){ #DK(ai,aj) = fK(ai) - fK(aj)
        DKf[i,j] <- PM[i,k] - PM[j,k]
      }
    }
    rownames(DKf) <- alt
    colnames(DKf) <- alt
    DK[[k]] <- DKf
  }
  names(DK) <- cri
  #print(DK) #DEBUG info

  #preference degree (validated in PROMETHEE)
  Pj <- list()
  for(k in 1:ncri){
    PjK <- matrix(data = 0, nrow = nalt, ncol = nalt)
    DKf <- DK[[k]]
    for(i in 1:nalt){
      for(j in 1:nalt){
        if(d[k] == 'default'){
          if(DKf[i,j] <= 0){
            PjK[i,j] <- 0
          }else{
            PjK[i,j] <- 1
          }
        }else if(d[k] == 'U-shape'){
          if(DKf[i,j] <= qj[k]){
            PjK[i,j] <- 0
          }else{
            PjK[i,j] <- 1
          }
        }else if(d[k] == 'V-shape'){
          if(DKf[i,j] <= 0){
            PjK[i,j] <- 0
          }else if(DKf[i,j] > 0 & DKf[i,j] <= pj[k]){
            PjK[i,j] <- DKf[i,j]/pj[k]
          }else{
            PjK[i,j] <- 1
          }
        }else if(d[k] == 'level'){
          if(DKf[i,j] <= qj[k]){
            PjK[i,j] <- 0
          }else if(DKf[i,j] > qj[k] & DKf[i,j] <= pj[k]){
            PjK[i,j] <- 0.5
          }else{
            PjK[i,j] <- 1
          }
        }else if(d[k] == 'linear'){
          if(DKf[i,j] <= qj[k]){
            PjK[i,j] <- 0
          }else if(DKf[i,j] > qj[k] & DKf[i,j] <= pj[k]){
            PjK[i,j] <- (DKf[i,j] - qj[k])/(pj[k] - qj[k])
          }else{
            PjK[i,j] <- 1
          }
        }else if(d[k] == 'Gaussian'){
          if(DKf[i,j] <= 0){
            PjK[i,j] <- 0
          }else{
            PjK[i,j] <- 1 - exp(-((DKf[i,j]*DKf[i,j])/(2*sj[k]*sj[k])))
          }
        }
      }
    }
    rownames(PjK) <- alt
    colnames(PjK) <- alt
    Pj[[k]] <- PjK
  }
  names(Pj) <- cri
  #print(Pj) #DEBUG info
  #TODO: oportunity - previous part of the code is same as in PROMETHEE function - refactor into separate
  #      function to be used by both SIR and PROMETHEE functions

  #superiority (Si) and inferiority (Ii) index
  Si <- matrix(0, nrow = nalt, ncol = ncri)
  Ii <- matrix(0, nrow = nalt, ncol = ncri)
  rownames(Si) <- alt
  rownames(Ii) <- alt
  colnames(Si) <- cri
  colnames(Ii) <- cri
  for(i in 1:ncri){
    PjK <- Pj[[i]]
    Si[,i] <- rowSums(PjK)
    Ii[,i] <- colSums(PjK)
  }

  #S-flow a I-flow
  if(SAW){
    #SIR SAW agregation method
    Sflow <- rowSums(sweep(Si, MARGIN=2, w, `*`))
    Iflow <- rowSums(sweep(Ii, MARGIN=2, w, `*`))
  }else{
    #SIR TOPSIS agregation method
    # as oposed to TOPSIS the procedure works separately with Si and Ii matrixes to derive S and I flow
    # Determination of the Ideal (A_ideal) and Anti-ideal (A_anti) Solutions (validated)
    # (as max or minimums in the criteria)
    Si <- as.data.frame(Si)
    Ii <- as.data.frame(Ii)
    A_ideal <- Si %>%
      group_by() %>%
      summarise(across(1:ncri, max)) %>%
      unlist()
    A_anti <- Si %>%
      group_by() %>%
      summarise(across(1:ncri, min)) %>%
      unlist()
    names(A_ideal) <- cri
    names(A_anti) <- cri

    #Step 4. Calculation of the Separation Measures (distance from ideal/anti ideal solution) (validated TOPSIS)
    D_ideal <- rep(0, times = nalt)
    D_anti <- rep(0, times = nalt)
    for(i in 1:nalt){
      for(j in 1:ncri){
        D_ideal[i] <- D_ideal[i] + (Si[i,j] - A_ideal[j])^2
        D_anti[i] <- D_anti[i] + (Si[i,j] - A_anti[j])^2
      }
    }
    D_ideal <- sqrt(D_ideal)
    D_anti <- sqrt(D_anti)
    names(D_ideal) <- alt
    names(D_anti) <- alt

    #Step 5. Calculation of the Relative Closeness to the Ideal Solution (validated TOPSIS)
    Sflow <- D_anti/(D_anti + D_ideal) #always 0-1, the closer to 1, the better

    # Determination of the Ideal (A_ideal) and Anti-ideal (A_anti) Solutions (validated TOPSIS)
    # (as max or minimums in the criteria)
    # note that for Ii max and min are switched - min for ideal and max for anti ideal solution
    A_ideal <- Ii %>%
      group_by() %>%
      summarise(across(1:ncri, min)) %>%
      unlist()
    A_anti <- Ii %>%
      group_by() %>%
      summarise(across(1:ncri, max)) %>%
      unlist()
    names(A_ideal) <- cri
    names(A_anti) <- cri

    #Step 4. Calculation of the Separation Measures (distance from ideal/anti ideal solution) (validated TOPSIS)
    D_ideal <- rep(0, times = nalt)
    D_anti <- rep(0, times = nalt)
    for(i in 1:nalt){
      for(j in 1:ncri){
        D_ideal[i] <- D_ideal[i] + (Ii[i,j] - A_ideal[j])^2
        D_anti[i] <- D_anti[i] + (Ii[i,j] - A_anti[j])^2
      }
    }
    D_ideal <- sqrt(D_ideal)
    D_anti <- sqrt(D_anti)
    names(D_ideal) <- alt
    names(D_anti) <- alt

    #Step 5. Calculation of the Relative Closeness to the Ideal Solution (validated TOPSIS)
    Iflow <- D_anti/(D_anti + D_ideal) #always 0-1, the closer to 1, the better
  }

  # first and second complete ranking (according to S and I flows)
  netFlow <- Sflow - Iflow
  relativeFlow <- Sflow / (Sflow + Iflow)
  flows <- cbind(Sflow, Iflow, netFlow, relativeFlow)
  rownames(flows) <- alt
  colnames(flows) <- c('S-flow', 'I-flow', 'n-flow', 'r-flow')
  names(Sflow) <- alt
  names(Iflow) <- alt
  names(netFlow) <- alt
  names(relativeFlow) <- alt
  Sflow <- sort(Sflow, decreasing = T)
  Iflow <- sort(Iflow, decreasing = F)
  netFlow <- sort(netFlow, decreasing = T)
  relativeFlow <- sort(relativeFlow, decreasing = T)
  rankSFlow <- rep(0, times=nalt)
  rankIFlow <- rep(0, times=nalt)
  names(rankSFlow) <- names(Sflow)
  names(rankIFlow) <- names(Iflow)
  for(i in 1:nalt){
    if(i == 1) {
      rankSFlow[i] <- 1
      rankIFlow[i] <- 1
    }else {
      if(Sflow[i-1] == Sflow[i]){
        rankSFlow[i] <- rankSFlow[i-1]
      }else{
        rankSFlow[i] <- rankSFlow[i-1] + 1
      }
      if(Iflow[i-1] == Iflow[i]){
        rankIFlow[i] <- rankIFlow[i-1]
      }else{
        rankIFlow[i] <- rankIFlow[i-1] + 1
      }
    }
  }

  # partial ranking
  partialRanking <- matrix(0, nrow = nalt, ncol = nalt)
  rownames(partialRanking) <- alt
  colnames(partialRanking) <- alt
  for(i in 1:nalt){
    for(j in 1:nalt){
      if(i == j){
        #no meaning evaluation alternative with itself
      }else if((rankSFlow[alt[i]] < rankSFlow[alt[j]] && rankIFlow[alt[i]] < rankIFlow[alt[j]]) ||
         (rankSFlow[alt[i]] < rankSFlow[alt[j]] && rankIFlow[alt[i]] == rankIFlow[alt[j]]) ||
         (rankSFlow[alt[i]] == rankSFlow[alt[j]] && rankIFlow[alt[i]] < rankIFlow[alt[j]])){
        partialRanking[i,j] <- 'P' #preference
      }else if(rankSFlow[alt[i]] == rankSFlow[alt[j]] && rankIFlow[alt[i]] == rankIFlow[alt[j]]){
        partialRanking[i,j] <- 'I' #indifference
      }else if(rankSFlow[alt[i]] < rankSFlow[alt[j]] && rankIFlow[alt[i]] > rankIFlow[alt[j]]){
        partialRanking[i,j] <- 'R' #incomparable
      }
    }
  }

  if(VERBOSE){
    print('Flows in the solution')
    print(flows)
  }

  out <- list(
    superiorityFlow = Sflow,
    inferiorityFlow = Iflow,
    rankSFlow = rankSFlow,
    rankIFlow = rankIFlow,
    partialRanking = partialRanking,
    netFlow = netFlow,
    relativeFlow = relativeFlow,
    flows = flows
  )
  return(out)
}
